"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const symbol_sdk_1 = require("symbol-sdk");
// internal dependencies
const index_1 = require("./NIP13/index");
const index_2 = require("../../index");
const SecuritiesMetadata_1 = require("./NIP13/models/SecuritiesMetadata");
const Accountable_1 = require("./NIP13/contracts/Accountable");
const MultisigService_1 = require("./NIP13/services/MultisigService");
const MetadataService_1 = require("./NIP13/services/MetadataService");
const PartitionService_1 = require("./NIP13/services/PartitionService");
const MosaicService_1 = require("./NIP13/services/MosaicService");
/**
 * @type NIP13.TokenMetadata
 * @package standards
 * @since v0.3.3
 * @description Class that describes NIP13 token metadata
 * @see {SecuritiesMetadata}
 */
class TokenMetadata extends SecuritiesMetadata_1.SecuritiesMetadata {
}
exports.TokenMetadata = TokenMetadata;
/**
 * @type NIP13.TokenAuthority
 * @package standards
 * @since v0.3.3
 * @description Class that describes NIP13 token authorities
 */
class TokenAuthority extends Accountable_1.Accountable {
    /**
     * Constructs a NIP13 token authority object.
     *
     * @param {NetworkConfig}       network
     * @param {MnemonicPassPhrase}  bip39
     */
    constructor(
    /**
     * @description The network configuration
     */
    network, 
    /**
     * @description The BIP39 mnemonic pass phrase used with said token.
     */
    bip39) {
        // @see {Accountable}
        super(network, bip39);
        this.network = network;
        this.bip39 = bip39;
        // derive base keys
        this.publicAccount = this.getAuthority().publicAccount;
    }
    /**
     * List a token authority's security tokens. A token authority
     * is used to prove the existance of said security tokens on
     * a network.
     *
     * @param {TokenAuthority} authority
     * @return {MosaicId[]}
     */
    static getTokens(authority, network) {
        return __awaiter(this, void 0, void 0, function* () {
            const account = authority instanceof symbol_sdk_1.PublicAccount
                ? authority
                : authority.publicAccount;
            // prepare context
            const context = new index_2.Context(exports.Revision, account, network, new index_2.TransactionParameters(), undefined);
            // use NIP13 mosaic service
            const service = new MosaicService_1.MosaicService(context);
            // prepare output
            const mosaics = yield service.getMosaicsFromNetwork(account, 'NIP13(v' + context.revision + '):create:' // mosaicId & name after this
            );
            return mosaics;
        });
    }
}
exports.TokenAuthority = TokenAuthority;
/**
 * @var NIP13.TokenCommands
 * @package standards
 * @since v0.1.0
 * @description Object that describes NIP13 available token commands
 * @link https://github.com/nemtech/NIP/blob/master/NIPs/nip-0013.md#token-commands
 */
exports.TokenCommands = {
    'CreateToken': (c, i, k) => new index_1.NIP13.CreateToken(c, i, k),
    'TransferOwnership': (c, i, k) => new index_1.NIP13.TransferOwnership(c, i, k),
};
/**
 * @var NIP13.Revision
 * @package standards
 * @since v0.1.0
 * @description Object that describes the count of NIP13 security token revisions
 */
exports.Revision = 1;
/**
 * @class NIP13.TokenStandard
 * @package standards
 * @since v0.1.0
 * @description Class that describes NIP13 security tokens standard
 * @link https://github.com/nemtech/NIP/blob/master/NIPs/nip-0013.md
 */
class TokenStandard extends Accountable_1.Accountable {
    /**
     * Constructs a NIP13 token standard object.
     *
     * @param {NetworkConfig}       network
     * @param {MnemonicPassPhrase}  bip39
     */
    constructor(
    /**
     * @description The network configuration
     */
    network, 
    /**
     * @description The BIP39 mnemonic pass phrase used with said token.
     */
    bip39) {
        // @see {Accountable}
        super(network, bip39);
        this.network = network;
        this.bip39 = bip39;
        /**
         * @description List of operators of said token.
         */
        this.operators = [];
        /**
         * @description Partition records of said token.
         */
        this.partitions = [];
        // derive base keys
        this.target = this.getTarget().publicAccount;
        // set network configuration
        this.source = new index_2.TokenSource(this.network.generationHash);
    }
    /**
     * Getter for the deterministic token identifier.
     *
     * @return {TokenIdentifier}
     */
    get identifier() {
        // prepare deterministic token identifier
        const hash = new Uint8Array(64);
        const data = this.target.address.plain() + '-' + this.source.source;
        symbol_sdk_1.SHA3Hasher.func(hash, symbol_sdk_1.Convert.utf8ToUint8(data), 64);
        // 4 left-most bytes for the identifier
        const left4b = hash.slice(0, 4).reduce((s, b) => s + b.toString(16).padStart(2, '0'), '');
        return new index_2.TokenIdentifier(left4b, this.source, this.target);
    }
    /**
     * Synchronize the command execution with the network. This method shall
     * be used to fetch data required for execution.
     *
     * @async
     * @return {Promise<boolean>}
     */
    synchronize() {
        return __awaiter(this, void 0, void 0, function* () {
            // prepare
            const target = this.target.address;
            const context = this.getContext(this.target, new index_2.TransactionParameters());
            const multisig = new MultisigService_1.MultisigService(context);
            const metadata = new MetadataService_1.MetadataService(context);
            const partitions = new PartitionService_1.PartitionService(context);
            // initialize REST
            const multisigHttp = context.network.factoryHttp.createMultisigRepository();
            const mosaicHttp = context.network.factoryHttp.createMosaicRepository();
            // consolidate/reduce graph
            const graph = yield multisigHttp.getMultisigAccountGraphInfo(target).toPromise();
            this.operators = multisig.getMultisigAccountInfoFromGraph(graph).map(m => m.cosignatories).reduce((prev, it) => prev.concat(it));
            // read mosaic
            this.mosaicInfo = yield mosaicHttp.getMosaic(this.identifier.toMosaicId()).toPromise();
            // read partitions
            this.partitions = yield partitions.getPartitionsFromNetwork(this.identifier, this.target, this.operators, 'NIP13(v' + context.revision + '):transfer:' + this.identifier.id + ':' // label after this
            );
            // read token metadata
            this.metadata = yield metadata.getMetadataFromNetwork(this.identifier);
            // success exit
            return true;
        });
    }
    /**
     * Creates a new NIP13 Token and configures operators.
     *
     * @param   {string}                  name
     * @param   {PublicAccount}           actor
     * @param   {PublicAccount} authority
     * @param   {PublicAccount}           target
     * @param   {Array<PublicAccount>}    operators
     * @param   {number}                  supply
     * @param   {TransactionParameters}   parameters
     * @return  {TokenIdentifier}
     **/
    create(authority, name, actor, operators, supply, metadata, parameters) {
        // generate deterministic token identifier
        const tokenId = this.identifier;
        // execute token command `CreateToken`
        this.result = this.execute(actor, tokenId, 'CreateToken', parameters, [
            new index_2.CommandOption('name', name),
            new index_2.CommandOption('source', this.source),
            new index_2.CommandOption('authority', authority),
            new index_2.CommandOption('operators', operators),
            new index_2.CommandOption('supply', supply),
            new index_2.CommandOption('metadata', metadata),
        ]);
        return tokenId;
    }
    /**
     * Transfer shared of a previously created Security Token with identifier `tokenId`.
     *
     * @internal This method MUST use the `TransferOwnership` command.
     * @param   {PublicAccount}         actor
     * @param   {PublicAccount}         partition
     * @param   {PublicAccount}         sender
     * @param   {PublicAccount}         recipient
     * @param   {string}                label
     * @param   {number}                amount
     * @param   {TransactionParameters} parameters
     * @return  {TransactionURI}
     **/
    transfer(actor, partition, sender, recipient, label, amount, parameters) {
        return __awaiter(this, void 0, void 0, function* () {
            // read state from REST API
            yield this.synchronize();
            // generate deterministic token identifier
            const tokenId = this.identifier;
            // execute token command `TransferOwnership`
            this.result = this.execute(actor, tokenId, 'TransferOwnership', parameters, [
                new index_2.CommandOption('name', label),
                new index_2.CommandOption('sender', sender),
                new index_2.CommandOption('partition', partition),
                new index_2.CommandOption('recipient', recipient),
                new index_2.CommandOption('amount', amount),
            ]);
            return this.result;
        });
    }
    /**
     * Notify an account `account` about `notification`
     *
     * @param   {TokenIdentifier} tokenId
     * @param   {PublicAccount}         account
     * @param   {Notification}    notification
     * @return  {NotificationProof}
     **/
    notify(tokenId, account, notification, parameters) {
        return new index_2.NotificationProof('');
    }
    /**
     * Verifies **allowance** of `sender` to transfer `tokenId` security token.
     *
     * @param   {PublicAccount}         sender
     * @param   {TokenIdentifier} tokenId
     * @return  {AllowanceResult}
     **/
    canTransfer(sender, tokenId) {
        return new index_2.AllowanceResult(true);
    }
    /**
     * Verifies **allowance** of `operator` to execute `command` with `tokenId` security token.
     *
     * @internal This method MUST use the `Command.canExecute()` method.
     * @param   {PublicAccount}         actor
     * @param   {TokenIdentifier}       tokenId
     * @param   {string}                command
     * @param   {Array<CommandOption>}  argv
     * @return  {AllowanceResult}
     **/
    canExecute(actor, tokenId, command, argv) {
        // instanciate command and context
        const params = new index_2.TransactionParameters();
        const context = this.getContext(actor, params, argv);
        const cmdFn = this.getCommand(tokenId, command, context);
        // populate async data
        cmdFn.mosaicInfo = this.mosaicInfo;
        cmdFn.operators = this.operators;
        cmdFn.partitions = this.partitions;
        // use `canExecute` for token command
        return cmdFn.canExecute(actor, argv);
    }
    /**
     * Execute `command` for Security Token with identifier `tokenId`. Arguments
     * the command execution can be passed in `argv`.
     *
     * @internal This method MUST use the `Command.execute()` method.
     * @param   {PublicAccount}         actor  The execution `actor`
     * @param   {TokenIdentifier}       tokenId
     * @param   {string}                command
     * @param   {Array<CommandOption>}  argv
     * @return  {TransactionURI}
     **/
    execute(actor, tokenId, command, parameters, argv) {
        try {
            // instanciate command and context
            const context = this.getContext(actor, parameters, argv);
            const cmdFn = this.getCommand(tokenId, command, context);
            // populate async data
            cmdFn.mosaicInfo = this.mosaicInfo;
            cmdFn.operators = this.operators;
            cmdFn.partitions = this.partitions;
            // execute token command
            return cmdFn.execute(actor, argv);
        }
        catch (f) {
            // XXX error notifications / events
            throw f;
        }
    }
    /**
     * Gets an execution context
     *
     * @param   {PublicAccount}           actor
     * @param   {TransactionParameters}   parameters
     * @param   {CommandOption[]}         argv
     * @return  {Context}
     */
    getContext(actor, parameters, argv) {
        return new index_2.Context(exports.Revision, actor, this.network, parameters, argv);
    }
    /**
     * Gets a command instance around `context` and `tokenId`.
     *
     * @param {TokenIdentifier} tokenId
     * @param {string}          command
     * @param {Context}         context
     * @return {Command}
     */
    getCommand(tokenId, command, context) {
        // validate token command
        if (!exports.TokenCommands || !exports.TokenCommands[command]) {
            throw new index_2.FailureInvalidCommand('Invalid token command.');
        }
        return exports.TokenCommands[command](context, tokenId, this.keyProvider);
    }
}
exports.TokenStandard = TokenStandard;
/**
 * @class NIP13.Token
 * @package standards
 * @since v0.1.0
 * @description Class that describes NIP13 compliant security tokens
 * @link https://github.com/nemtech/NIP/blob/master/NIPs/nip-0013.md
 */
class Token extends TokenStandard {
}
exports.Token = Token;
