/**
 * Copyright 2020 NEM
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
import { PublicAccount, Transaction } from 'symbol-sdk';
import { CommandOption, AllowanceResult } from '../../../../index';
import { AbstractCommand } from './AbstractCommand';
/**
 * @class NIP13.CreateToken
 * @package NIP13 Token Commands
 * @since v0.1.0
 * @description Class that describes a token command for creating NIP13 compliant tokens.
 * @transactions This token command prepares one aggregate bonded transaction with following inner transactions:
 *   - Transaction 01: MultisigAccountModificationTransaction
 *   - Transaction 02: AccountMetadataTransaction attaching `NIP13` token identifier
 *   - Transaction 03: NamespaceRegistrationtTransaction
 *   - Transaction 04: MosaicDefinitionTransaction
 *   - Transaction 05: MosaicSupplyChangeTransaction
 *   - Transaction 06: MosaicAliasTransaction
 *   - Transaction 07: MosaicMetadataTransaction attaching `NIP13` token identifier
 *   - Transaction 08: MosaicMetadataTransaction attaching `NAME`
 *   - Transaction 09: MosaicMetadataTransaction attaching `MIC` market identifier code
 *   - Transaction 10: MosaicMetadataTransaction attaching `ISIN` if non-empty option provided
 *   - Transaction 11: MosaicMetadataTransaction attaching `ISO_10962` if non-empty option provided
 *   - Transaction 12: MosaicMetadataTransaction attaching custom metadata if non-empty option provided
 *   - Transaction 13: AccountMosaicRestrictionTransaction with targets = [mosaicId, feeMosaicId] to allow mosaic and fees for TARGET account
 *   - Transaction 14: MosaicGlobalRestriction with mosaicId (refId 0) (User_Role <= 3)
 *   - Transaction 15: MosaicAddressRestriction for target account (User_Role = Target)
 *   - Transaction 16: TransferTransaction with NIP13 `CreateToken` command descriptor (Transfer to `authority` account)
 *
 * :note: `Transaction 04` represents the root namespace registration transaction. Any sub namespace registration
 * transaction will be automatically added to this list.
 *
 * :note: `Transaction 10` will only be added if a non-empty value is provided for the `source` command option.
 *
 * :note: `Transaction 11` will only be added if a non-empty value is provided for the `isin` command option.
 *
 * :note: `Transaction 12` will only be added if a non-empty value is provided for the `iso10962` command option.
 *
 * :note: `Transaction 13` will only be added if a non-empty value is provided for custom metadata fields.
 */
export declare class CreateToken extends AbstractCommand {
    /**
     * @description List of **required** arguments for this token command.
     * @link https://en.wikipedia.org/wiki/International_Securities_Identification_Number
     * @link https://en.wikipedia.org/wiki/ISO_10962
     * @link https://en.wikipedia.org/wiki/Market_Identifier_Code
     */
    arguments: string[];
    /**
     * Synchronize the command execution with the network. This method shall
     * be used to fetch data required for execution.
     *
     * @async
     * @override The 'CreateToken' command does not need synchronization.
     * @return {Promise<boolean>}
     */
    synchronize(): Promise<boolean>;
    /**
     * @description Method that verifies the allowance of an operator to
     *              execute the token command `CreateToken`.
     * @see {BaseCommand.canExecute}
     **/
    canExecute(actor: PublicAccount, argv?: CommandOption[]): AllowanceResult;
    /**
     * @description Getter for the command name.
     * @see {AbstractCommand.name}
     **/
    get name(): string;
    /**
     * Getter for the command descriptor.
     *
     * @return {string}
     **/
    get descriptor(): string;
    /**
     * @description Builds the inner transactions necessary for the
     *              execution of a `CreateToken` command.
     * @see {AbstractCommand.transactions}
     * @return {Transaction[]} Aggregate bonded transaction
     **/
    protected get transactions(): Transaction[];
}
