/**
 * Copyright 2020 NEM
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
import { InnerTransaction, Transaction, PublicAccount } from 'symbol-sdk';
import { AbstractCommand } from './AbstractCommand';
/**
 * @class NIP13.TransferOwnership
 * @package NIP13 Token Commands
 * @since v0.1.0
 * @description Class that describes a token command for transferring NIP13 compliant tokens.
 * @summary This token command prepares one aggregate bonded transaction with following inner transactions:
 *
 * 1) sending from **target** account, should create new partition multisig + transfer
 *   - Transaction 1.01: MultisigAccountModificationTransaction
 *   - Transaction 1.02: AccountMetadataTransaction attaching `NAME`
 *   - Transaction 1.03: AccountMosaicRestrictionTransaction for mosaicId & fee
 *   - Transaction 1.04: MosaicAddressRestriction with `User_Role = Holder` for partition
 *   - Transaction 1.05: Add ownership transfer transaction
 *
 * 2) sending from **partition** account (not target account), should change ownership
 *    from **owner** to **recipient**
 *
 *   2.1) In case the new recipient is another partition, the amount must be sent back to
 *        the target account, then send to the new partition account.
 *   - Transaction 2.1.01: First send back the amount to the target account
 *   - Transaction 2.1.02: Add ownership transfer transaction
 *
 *   2.2) In case the new recipient is not a partition, a new partition will be created and
 *        the amount will be transferred to it.
 *   - Transaction 2.2.01: MultisigAccountModificationTransaction
 *   - Transaction 2.2.02: AccountMetadataTransaction attaching `NAME`
 *   - Transaction 2.2.03: AccountMosaicRestrictionTransaction for mosaicId & fee
 *   - Transaction 2.2.04: MosaicAddressRestriction with `User_Role = Holder` for partition
 *   - Transaction 2.2.05: First send back the amount to the target account
 *   - Transaction 2.2.06: Add ownership transfer transaction
 */
export declare class TransferOwnership extends AbstractCommand {
    /**
     * @description List of **required** arguments for this token command.
     */
    arguments: string[];
    /**
     * @description Getter for the command name.
     * @see {BaseCommand.name}
     **/
    get name(): string;
    /**
     * Getter for the command descriptor.
     *
     * @return {string}
     **/
    get descriptor(): string;
    /**
     * @description Builds the inner transactions necessary for the
     *              execution of a `TransferOwnership` command.
     * @see {AbstractCommand.transactions}
     * @return {Transaction[]} Aggregate bonded transaction
     **/
    protected get transactions(): Transaction[];
    /**
     * Helper method for creating partitions on Symbol networks.
     *
     * @param {PublicAccount} partition
     * @param {PublicAccount} recipient
     * @param {string}        name
     */
    protected createPartition(partition: PublicAccount, recipient: PublicAccount, name: string): {
        transactions: InnerTransaction[];
        signers: PublicAccount[];
    };
}
